from __future__ import unicode_literals

from django.apps import apps
from django.contrib.contenttypes.fields import GenericRelation
from django.db import models


class ErrorLogEntryManager(models.Manager):
    def register(self, model):
        ErrorLogEntry = apps.get_model(
            app_label='common', model_name='ErrorLogEntry'
        )
        model.add_to_class('error_logs', GenericRelation(ErrorLogEntry))



from __future__ import unicode_literals

from django.test import override_settings
from django.utils.encoding import force_text

from common.tests import BaseTestCase
from documents.models import DocumentType
from documents.tests import TEST_SMALL_DOCUMENT_PATH, TEST_DOCUMENT_TYPE_LABEL
from metadata.models import MetadataType, DocumentTypeMetadataType

from ..models import Index, IndexInstanceNode, IndexTemplateNode

from .literals import (
    TEST_INDEX_LABEL, TEST_INDEX_TEMPLATE_METADATA_EXPRESSION,
    TEST_METADATA_TYPE_LABEL, TEST_METADATA_TYPE_NAME
)


@override_settings(OCR_AUTO_OCR=False)
class IndexTestCase(BaseTestCase):
    def setUp(self):
        super(IndexTestCase, self).setUp()
        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE_LABEL
        )

        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=file_object
            )

    def tearDown(self):
        self.document_type.delete()
        super(IndexTestCase, self).tearDown()

    def test_indexing(self):
        metadata_type = MetadataType.objects.create(
            name=TEST_METADATA_TYPE_NAME, label=TEST_METADATA_TYPE_LABEL
        )
        DocumentTypeMetadataType.objects.create(
            document_type=self.document_type, metadata_type=metadata_type
        )

        # Create empty index
        index = Index.objects.create(label=TEST_INDEX_LABEL)

        # Add our document type to the new index
        index.document_types.add(self.document_type)

        # Create simple index template
        root = index.template_root
        index.node_templates.create(
            parent=root, expression=TEST_INDEX_TEMPLATE_METADATA_EXPRESSION,
            link_documents=True
        )

        # Add document metadata value to trigger index node instance creation
        self.document.metadata.create(
            metadata_type=metadata_type, value='0001'
        )
        self.assertEqual(
            list(
                IndexInstanceNode.objects.values_list('value', flat=True)
            ), ['', '0001']
        )

        # Check that document is in instance node
        instance_node = IndexInstanceNode.objects.get(value='0001')
        self.assertQuerysetEqual(
            instance_node.documents.all(), [repr(self.document)]
        )

        # Change document metadata value to trigger index node instance update
        document_metadata = self.document.metadata.get(
            metadata_type=metadata_type
        )
        document_metadata.value = '0002'
        document_metadata.save()
        self.assertEqual(
            list(
                IndexInstanceNode.objects.values_list('value', flat=True)
            ), ['', '0002']
        )

        # Check that document is in new instance node
        instance_node = IndexInstanceNode.objects.get(value='0002')
        self.assertQuerysetEqual(
            instance_node.documents.all(), [repr(self.document)]
        )

        # Check node instance is destoyed when no metadata is available
        self.document.metadata.get(metadata_type=metadata_type).delete()
        self.assertEqual(
            list(
                IndexInstanceNode.objects.values_list('value', flat=True)
            ), ['']
        )

        # Add document metadata value again to trigger index node instance
        # creation
        self.document.metadata.create(
            metadata_type=metadata_type, value='0003'
        )
        self.assertEqual(
            list(
                IndexInstanceNode.objects.values_list('value', flat=True)
            ), ['', '0003']
        )

        # Check node instance is destroyed when no documents are contained
        self.document.delete()

        # Document is in trash, index structure should remain unchanged
        self.assertEqual(
            list(
                IndexInstanceNode.objects.values_list('value', flat=True)
            ), ['', '0003']
        )

        # Document deleted, index structure should update
        self.document.delete()
        self.assertEqual(
            list(
                IndexInstanceNode.objects.values_list('value', flat=True)
            ), ['']
        )

    def test_rebuild_all_indexes(self):
        # Add metadata type and connect to document type
        metadata_type = MetadataType.objects.create(name='test', label='test')
        DocumentTypeMetadataType.objects.create(
            document_type=self.document_type, metadata_type=metadata_type
        )

        # Add document metadata value
        self.document.metadata.create(
            metadata_type=metadata_type, value='0001'
        )

        # Create empty index
        index = Index.objects.create(label='test')
        self.assertEqual(list(Index.objects.all()), [index])

        # Add our document type to the new index
        index.document_types.add(self.document_type)
        self.assertQuerysetEqual(
            index.document_types.all(), [repr(self.document_type)]
        )

        # Create simple index template
        root = index.template_root
        index.node_templates.create(
            parent=root, expression='{{ document.metadata_value_of.test }}',
            link_documents=True
        )
        self.assertEqual(
            list(
                IndexTemplateNode.objects.values_list('expression', flat=True)
            ), ['', '{{ document.metadata_value_of.test }}']
        )

        # There should be no index instances
        self.assertEqual(list(IndexInstanceNode.objects.all()), [])

        # Rebuild all indexes
        Index.objects.rebuild()

        # Check that document is in instance node
        instance_node = IndexInstanceNode.objects.get(value='0001')
        self.assertQuerysetEqual(
            instance_node.documents.all(), [repr(self.document)]
        )

    def test_dual_level_dual_document_index(self):
        """
        Test creation of an index instance with two first levels with different
        values and two second levels with the same value but as separate
        children of each of the first levels. GitLab issue #391
        """
        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document_2 = self.document_type.new_document(
                file_object=file_object
            )

        # Create empty index
        index = Index.objects.create(label=TEST_INDEX_LABEL)

        # Add our document type to the new index
        index.document_types.add(self.document_type)

        # Create simple index template
        root = index.template_root
        level_1 = index.node_templates.create(
            parent=root, expression='{{ document.uuid }}',
            link_documents=False
        )

        index.node_templates.create(
            parent=level_1, expression='{{ document.label }}',
            link_documents=True
        )

        Index.objects.rebuild()

        self.assertEqual(
            [instance.value for instance in IndexInstanceNode.objects.all().order_by('pk')],
            [
                '', force_text(self.document_2.uuid), self.document_2.label,
                force_text(self.document.uuid), self.document.label
            ]
        )

    def test_multi_level_template_with_no_result_parent(self):
        """
        On a two level template if the first level doesn't return a result
        the indexing should stop. GitLab issue #391.
        """
        index = Index.objects.create(label=TEST_INDEX_LABEL)
        index.document_types.add(self.document_type)

        level_1 = index.node_templates.create(
            parent=index.template_root,
            expression='',
            link_documents=True
        )

        index.node_templates.create(
            parent=level_1, expression='{{ document.label }}',
            link_documents=True
        )

        Index.objects.rebuild()



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from navigation import Link

from .permissions import permission_content_view, permission_parse_document

link_document_content = Link(
    args='resolved_object.id', icon='fa fa-font',
    permissions=(permission_content_view,), text=_('Content'),
    view='document_parsing:document_content',
)
link_document_parsing_errors_list = Link(
    args='resolved_object.id', icon='fa fa-file-text-o',
    permissions=(permission_content_view,), text=_('Parsing errors'),
    view='document_parsing:document_parsing_error_list'
)
link_document_content_download = Link(
    args='resolved_object.id', icon='fa fa-file-text-o',
    permissions=(permission_content_view,), text=_('Download content'),
    view='document_parsing:document_content_download'
)
link_document_submit_multiple = Link(
    text=_('Submit for parsing'),
    view='document_parsing:document_submit_multiple'
)
link_document_submit = Link(
    args='resolved_object.id', permissions=(permission_parse_document,),
    text=_('Submit for parsing'), view='document_parsing:document_submit'
)
link_document_type_submit = Link(
    icon='fa fa-crosshairs', text=_('Parse documents per type'),
    view='document_parsing:document_type_submit'
)
link_error_list = Link(
    icon='fa fa-file-text-o', permissions=(permission_content_view,),
    text=_('Parsing errors'), view='document_parsing:error_list'
)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from smart_settings import Namespace

namespace = Namespace(name='sources', label=_('Sources'))

setting_scanimage_path = namespace.add_setting(
    global_name='SOURCE_SCANIMAGE_PATH', default='/usr/bin/scanimage',
    help_text=_(
        'File path to the scanimage program used to control image scanners.'
    ),
    is_path=True
)



from __future__ import unicode_literals

from django.apps import apps

from mayan.celery import app


@app.task(ignore_result=True)
def task_send_document(subject_text, body_text_content, sender, recipient, user_mailer_id, as_attachment=False, document_id=None):
    Document = apps.get_model(
        app_label='documents', model_name='Document'
    )
    UserMailer = apps.get_model(
        app_label='mailer', model_name='UserMailer'
    )

    if document_id:
        document = Document.objects.get(pk=document_id)
    else:
        document = None

    user_mailer = UserMailer.objects.get(pk=user_mailer_id)

    user_mailer.send(
        subject=subject_text, body=body_text_content, to=recipient,
        document=document, as_attachment=as_attachment
    )



from __future__ import unicode_literals

from django.core.exceptions import PermissionDenied

from common.tests import BaseTestCase

from ..classes import Permission
from ..permissions import permission_role_view


class PermissionTestCase(BaseTestCase):
    def setUp(self):
        super(PermissionTestCase, self).setUp()

    def test_no_permissions(self):
        with self.assertRaises(PermissionDenied):
            Permission.check_permissions(
                requester=self.user, permissions=(permission_role_view,)
            )

    def test_with_permissions(self):
        self.group.user_set.add(self.user)
        self.role.permissions.add(permission_role_view.stored_permission)
        self.role.groups.add(self.group)

        try:
            Permission.check_permissions(
                requester=self.user, permissions=(permission_role_view,)
            )
        except PermissionDenied:
            self.fail('PermissionDenied exception was not expected.')



from __future__ import unicode_literals

import os

from django.conf import settings
from django.core import management
from django.core.management.utils import get_random_secret_key

from ...settings import setting_local_settings_filename

from .literals import SETTING_FILE_TEMPLATE


class Command(management.BaseCommand):
    help = 'Creates a local settings file with a random secret key.'

    def handle(self, *args, **options):
        path = os.path.join(settings.BASE_DIR, 'settings', '{}.py'.format(setting_local_settings_filename.value))
        if os.path.exists(path):
            self.stdout.write(self.style.NOTICE('Existing settings file at: {0}. Backup, remove this file, and try again.'.format(path)))
        else:
            with open(path, 'w+') as file_object:
                file_object.write(
                    SETTING_FILE_TEMPLATE.format(get_random_secret_key())
                )



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from permissions import PermissionNamespace

namespace = PermissionNamespace('common', _('Common'))

permission_error_log_view = namespace.add_permission(
    name='error_log_view', label=_('View error log')
)



from __future__ import absolute_import, unicode_literals

from django.core import mail

from documents.tests.test_models import GenericDocumentTestCase

from ..models import UserMailer

from .literals import (
    TEST_EMAIL_ADDRESS, TEST_RECIPIENTS_MULTIPLE_COMMA,
    TEST_RECIPIENTS_MULTIPLE_SEMICOLON, TEST_RECIPIENTS_MULTIPLE_MIXED,
    TEST_RECIPIENTS_MULTIPLE_MIXED_LIST, TEST_USER_MAILER_LABEL,
    TEST_USER_MAILER_BACKEND_PATH
)


class ModelTestCase(GenericDocumentTestCase):
    def _create_user_mailer(self):
        self.user_mailer = UserMailer.objects.create(
            default=True,
            enabled=True,
            label=TEST_USER_MAILER_LABEL,
            backend_path=TEST_USER_MAILER_BACKEND_PATH,
            backend_data='{}'
        )

    def test_send_simple(self):
        self._create_user_mailer()
        self.user_mailer.send(to=TEST_EMAIL_ADDRESS)

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].to, [TEST_EMAIL_ADDRESS])

    def test_send_attachment(self):
        self._create_user_mailer()
        self.user_mailer.send(
            to=TEST_EMAIL_ADDRESS, document=self.document, as_attachment=True
        )

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].to, [TEST_EMAIL_ADDRESS])
        with self.document.open() as file_object:
            self.assertEqual(
                mail.outbox[0].attachments[0], (
                    self.document.label, file_object.read(),
                    self.document.file_mimetype
                )
            )

    def test_send_multiple_recipients_comma(self):
        self._create_user_mailer()
        self.user_mailer.send(to=TEST_RECIPIENTS_MULTIPLE_COMMA)

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(
            mail.outbox[0].to, TEST_RECIPIENTS_MULTIPLE_COMMA.split(',')
        )

    def test_send_multiple_recipients_semicolon(self):
        self._create_user_mailer()
        self.user_mailer.send(to=TEST_RECIPIENTS_MULTIPLE_SEMICOLON)

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(
            mail.outbox[0].to, TEST_RECIPIENTS_MULTIPLE_SEMICOLON.split(';')
        )

    def test_send_multiple_recipient_mixed(self):
        self._create_user_mailer()
        self.user_mailer.send(to=TEST_RECIPIENTS_MULTIPLE_MIXED)

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(
            list(mail.outbox[0].to), list(TEST_RECIPIENTS_MULTIPLE_MIXED_LIST)
        )



from __future__ import unicode_literals

import inspect
import logging

from furl import furl

from django.apps import apps
from django.conf import settings
from django.core.exceptions import PermissionDenied
from django.shortcuts import resolve_url
from django.template import VariableDoesNotExist, Variable
from django.template.defaulttags import URLNode
from django.urls import resolve
from django.utils.encoding import force_str, force_text

from common.utils import return_attrib
from permissions import Permission

logger = logging.getLogger(__name__)


class ResolvedLink(object):
    def __init__(self, link, current_view):
        self.current_view = current_view
        self.disabled = False
        self.link = link
        self.url = '#'
        self.context = None
        self.request = None

    @property
    def active(self):
        return self.link.view == self.current_view

    @property
    def description(self):
        return self.link.description

    @property
    def icon(self):
        return self.link.icon

    @property
    def tags(self):
        return self.link.tags

    @property
    def text(self):
        try:
            return self.link.text(context=self.context)
        except TypeError:
            return self.link.text


class Menu(object):
    _registry = {}

    @classmethod
    def get(cls, name):
        return cls._registry[name]

    @classmethod
    def remove(cls, name):
        del cls._registry[name]

    def __init__(self, name, icon=None, label=None):
        if name in self.__class__._registry:
            raise Exception('A menu with this name already exists')

        self.icon = icon
        self.name = name
        self.label = label
        self.bound_links = {}
        self.unbound_links = {}
        self.link_positions = {}
        self.__class__._registry[name] = self

    def _map_links_to_source(self, links, source, map_variable='bound_links', position=None):
        source_links = getattr(self, map_variable).setdefault(source, [])

        for link in links:
            source_links.append(link)
            self.link_positions[link] = position

    def bind_links(self, links, sources=None, position=None):
        """
        Associate a link to a model, a view inside this menu
        """

        try:
            for source in sources:
                self._map_links_to_source(
                    links=links, position=position, source=source
                )
        except TypeError:
            # Unsourced links display always
            self._map_links_to_source(
                links=links, position=position, source=sources
            )

    def get_resolved_navigation_object_list(self, context, source):
        resolved_navigation_object_list = []

        if source:
            resolved_navigation_object_list = [source]
        else:
            navigation_object_list = context.get(
                'navigation_object_list', ('object',)
            )

            logger.debug('navigation_object_list: %s', navigation_object_list)

            # Multiple objects
            for navigation_object in navigation_object_list:
                try:
                    resolved_navigation_object_list.append(
                        Variable(navigation_object).resolve(context)
                    )
                except VariableDoesNotExist:
                    pass

        logger.debug(
            'resolved_navigation_object_list: %s',
            force_text(resolved_navigation_object_list)
        )
        return resolved_navigation_object_list

    def resolve(self, context, source=None):
        result = []

        try:
            request = Variable('request').resolve(context)
        except VariableDoesNotExist:
            # There is no request variable, most probable a 500 in a test view
            # Don't return any resolved links then.
            logger.warning('No request variable, aborting menu resolution')
            return ()

        current_path = request.META['PATH_INFO']

        # Get sources: view name, view objects
        current_view = resolve(current_path).view_name

        resolved_navigation_object_list = self.get_resolved_navigation_object_list(
            context=context, source=source
        )

        for resolved_navigation_object in resolved_navigation_object_list:
            resolved_links = []

            for bound_source, links in self.bound_links.items():
                try:
                    if inspect.isclass(bound_source):
                        if type(resolved_navigation_object) == bound_source:
                            for link in links:
                                resolved_link = link.resolve(
                                    context=context,
                                    resolved_object=resolved_navigation_object
                                )
                                if resolved_link:
                                    if resolved_link.link not in self.unbound_links.get(bound_source, ()):
                                        resolved_links.append(resolved_link)
                            # No need for further content object match testing
                            break
                        elif hasattr(resolved_navigation_object, 'get_deferred_fields') and resolved_navigation_object.get_deferred_fields() and isinstance(resolved_navigation_object, bound_source):
                            # Second try for objects using .defer() or .only()
                            for link in links:
                                resolved_link = link.resolve(
                                    context=context,
                                    resolved_object=resolved_navigation_object
                                )
                                if resolved_link:
                                    if resolved_link.link not in self.unbound_links.get(bound_source, ()):
                                        resolved_links.append(resolved_link)
                            # No need for further content object match testing
                            break
                except TypeError:
                    # When source is a dictionary
                    pass

            if resolved_links:
                result.append(resolved_links)

        resolved_links = []
        # View links
        for link in self.bound_links.get(current_view, []):
            resolved_link = link.resolve(context=context)
            if resolved_link:
                if resolved_link.link not in self.unbound_links.get(current_view, ()):
                    resolved_links.append(resolved_link)

        if resolved_links:
            result.append(resolved_links)

        resolved_links = []

        # Main menu links
        for link in self.bound_links.get(None, []):
            if isinstance(link, Menu):
                resolved_link = link
                if resolved_link not in self.unbound_links.get(None, ()):
                    resolved_links.append(resolved_link)
            else:
                # "Always show" links
                resolved_link = link.resolve(context=context)
                if resolved_link:
                    if resolved_link.link not in self.unbound_links.get(None, ()):
                        resolved_links.append(resolved_link)

        if resolved_links:
            result.append(resolved_links)

        if result:
            # Sort links by position value passed during bind
            result[0] = sorted(
                result[0], key=lambda item: (self.link_positions.get(item.link) or 0) if isinstance(item, ResolvedLink) else (self.link_positions.get(item) or 0)
            )

        return result

    def unbind_links(self, links, sources=None):
        """
        Allow unbinding links from sources, used to allow 3rd party apps to
        change the link binding of core apps
        """

        try:
            for source in sources:
                self._map_links_to_source(
                    links=links, source=source, map_variable='unbound_links'
                )
        except TypeError:
            # Unsourced links display always
            self._map_links_to_source(
                links=links, source=sources, map_variable='unbound_links'
            )


class Link(object):
    def __init__(self, text, view=None, args=None, condition=None,
                 conditional_disable=None, description=None, icon=None,
                 keep_query=False, kwargs=None, permissions=None,
                 permissions_related=None, remove_from_query=None, tags=None,
                 url=None):

        self.args = args or []
        self.condition = condition
        self.conditional_disable = conditional_disable
        self.description = description
        self.icon = icon
        self.keep_query = keep_query
        self.kwargs = kwargs or {}
        self.permissions = permissions or []
        self.permissions_related = permissions_related
        self.remove_from_query = remove_from_query or []
        self.tags = tags
        self.text = text
        self.view = view
        self.url = url

    def resolve(self, context, resolved_object=None):
        AccessControlList = apps.get_model(
            app_label='acls', model_name='AccessControlList'
        )

        request = Variable('request').resolve(context)
        current_path = request.META['PATH_INFO']
        current_view = resolve(current_path).view_name

        # ACL is tested agains the resolved_object or just {{ object }} if not
        if not resolved_object:
            try:
                resolved_object = Variable('object').resolve(context=context)
            except VariableDoesNotExist:
                pass

        # If this link has a required permission check that the user has it
        # too
        if self.permissions:
            if resolved_object:
                try:
                    AccessControlList.objects.check_access(
                        permissions=self.permissions, user=request.user,
                        obj=resolved_object, related=self.permissions_related
                    )
                except PermissionDenied:
                    return None
            else:
                try:
                    Permission.check_permissions(
                        requester=request.user, permissions=self.permissions
                    )
                except PermissionDenied:
                    return None

        # Check to see if link has conditional display function and only
        # display it if the result of the conditional display function is
        # True
        if self.condition:
            if not self.condition(context):
                return None

        resolved_link = ResolvedLink(current_view=current_view, link=self)

        if self.view:
            view_name = Variable('"{}"'.format(self.view))
            if isinstance(self.args, list) or isinstance(self.args, tuple):
                # TODO: Don't check for instance check for iterable in try/except
                # block. This update required changing all 'args' argument in
                # links.py files to be iterables and not just strings.
                args = [Variable(arg) for arg in self.args]
            else:
                args = [Variable(self.args)]

            # If we were passed an instance of the view context object we are
            # resolving, inject it into the context. This help resolve links for
            # object lists.
            if resolved_object:
                context['resolved_object'] = resolved_object

            try:
                kwargs = self.kwargs(context)
            except TypeError:
                # Is not a callable
                kwargs = self.kwargs

            kwargs = {key: Variable(value) for key, value in kwargs.items()}

            # Use Django's exact {% url %} code to resolve the link
            node = URLNode(
                view_name=view_name, args=args, kwargs=kwargs, asvar=None
            )
            try:
                resolved_link.url = node.render(context)
            except Exception as exception:
                logger.error(
                    'Error resolving link "%s" URL; %s', self.text, exception
                )
        elif self.url:
            resolved_link.url = self.url

        # This is for links that should be displayed but that are not clickable
        if self.conditional_disable:
            resolved_link.disabled = self.conditional_disable(context)
        else:
            resolved_link.disabled = False

        # Lets a new link keep the same URL query string of the current URL
        if self.keep_query:
            # Sometimes we are required to remove a key from the URL QS
            parsed_url = furl(
                force_str(
                    request.get_full_path() or request.META.get(
                        'HTTP_REFERER', resolve_url(settings.LOGIN_REDIRECT_URL)
                    )
                )
            )

            for key in self.remove_from_query:
                try:
                    parsed_url.query.remove(key)
                except KeyError:
                    pass

            resolved_link.url = parsed_url.url

        resolved_link.context = context
        return resolved_link


class Separator(Link):
    """
    Menu separator. Renders to an <hr> tag
    """
    def __init__(self, *args, **kwargs):
        self.icon = None
        self.text = None
        self.view = None

    def resolve(self, *args, **kwargs):
        result = ResolvedLink(current_view=None, link=self)
        result.separator = True
        return result


class SourceColumn(object):
    _registry = {}

    @staticmethod
    def sort(columns):
        return sorted(columns, key=lambda x: x.order)

    @classmethod
    def get_for_source(cls, source):
        try:
            return SourceColumn.sort(columns=cls._registry[source])
        except KeyError:
            try:
                # Try it as a queryset
                return SourceColumn.sort(columns=cls._registry[source.model])
            except AttributeError:
                try:
                    # It seems to be an instance, try its class
                    return SourceColumn.sort(columns=cls._registry[source.__class__])
                except KeyError:
                    try:
                        # Special case for queryset items produced from
                        # .defer() or .only() optimizations
                        return SourceColumn.sort(columns=cls._registry[source._meta.parents.items()[0][0]])
                    except (AttributeError, KeyError, IndexError):
                        return ()
        except TypeError:
            # unhashable type: list
            return ()

    def __init__(self, source, label, attribute=None, func=None, order=None):
        self.source = source
        self.label = label
        self.attribute = attribute
        self.func = func
        self.order = order or 0
        self.__class__._registry.setdefault(source, [])
        self.__class__._registry[source].append(self)

    def resolve(self, context):
        if self.attribute:
            result = return_attrib(context['object'], self.attribute)
        elif self.func:
            result = self.func(context=context)

        return result


class Text(Link):
    """
    Menu text. Renders to a plain <li> tag
    """
    def __init__(self, *args, **kwargs):
        self.icon = None
        self.text = kwargs.get('text')
        self.view = None

    def resolve(self, *args, **kwargs):
        result = ResolvedLink(current_view=None, link=self)
        result.context = kwargs.get('context')
        result.text_span = True
        return result



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from smart_settings import Namespace

from .literals import DEFAULT_MAXIMUM_TITLE_LENGTH

namespace = Namespace(name='appearance', label=_('Appearance'))
setting_max_title_length = namespace.add_setting(
    global_name='APPEARANCE_MAXIMUM_TITLE_LENGTH',
    default=DEFAULT_MAXIMUM_TITLE_LENGTH
)



from __future__ import unicode_literals

from django.apps import apps
from django.db.models.signals import post_save
from django.utils.translation import ugettext_lazy as _

from kombu import Exchange, Queue

from acls import ModelPermission
from acls.links import link_acl_list
from common import (
    MayanAppConfig, menu_facet, menu_main, menu_object, menu_secondary,
    menu_setup, menu_sidebar, menu_tools
)
from common.classes import ModelAttribute
from common.links import link_object_error_list
from common.permissions_runtime import permission_error_log_view
from common.widgets import two_state_template
from mayan.celery import app
from navigation import SourceColumn
from rest_api.classes import APIEndPoint

from .classes import DocumentStateHelper, WorkflowAction
from .handlers import (
    handler_index_document, handler_trigger_transition, launch_workflow
)
from .links import (
    link_document_workflow_instance_list, link_setup_workflow_document_types,
    link_setup_workflow_create, link_setup_workflow_delete,
    link_setup_workflow_edit, link_setup_workflow_list,
    link_setup_workflow_states, link_setup_workflow_state_action_delete,
    link_setup_workflow_state_action_edit,
    link_setup_workflow_state_action_list,
    link_setup_workflow_state_action_selection,
    link_setup_workflow_state_create, link_setup_workflow_state_delete,
    link_setup_workflow_state_edit, link_setup_workflow_transitions,
    link_setup_workflow_transition_create,
    link_setup_workflow_transition_delete, link_setup_workflow_transition_edit,
    link_tool_launch_all_workflows, link_workflow_instance_detail,
    link_workflow_instance_transition, link_workflow_document_list,
    link_workflow_list, link_workflow_preview,
    link_workflow_state_document_list, link_workflow_state_list,
    link_workflow_instance_transition_events
)
from .permissions import (
    permission_workflow_delete, permission_workflow_edit,
    permission_workflow_transition, permission_workflow_view
)
from .queues import *  # NOQA
from .widgets import widget_transition_events


class DocumentStatesApp(MayanAppConfig):
    app_url = 'states'
    has_tests = True
    name = 'document_states'
    verbose_name = _('Document states')

    def ready(self):
        super(DocumentStatesApp, self).ready()

        APIEndPoint(app=self, version_string='1')

        Action = apps.get_model(
            app_label='actstream', model_name='Action'
        )
        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )
        ErrorLogEntry = apps.get_model(
            app_label='common', model_name='ErrorLogEntry'
        )

        Workflow = self.get_model('Workflow')
        WorkflowInstance = self.get_model('WorkflowInstance')
        WorkflowInstanceLogEntry = self.get_model('WorkflowInstanceLogEntry')
        WorkflowRuntimeProxy = self.get_model('WorkflowRuntimeProxy')
        WorkflowState = self.get_model('WorkflowState')
        WorkflowStateAction = self.get_model('WorkflowStateAction')
        WorkflowStateRuntimeProxy = self.get_model('WorkflowStateRuntimeProxy')
        WorkflowTransition = self.get_model('WorkflowTransition')
        WorkflowTransitionTriggerEvent = self.get_model(
            'WorkflowTransitionTriggerEvent'
        )

        Document.add_to_class(
            'workflow', DocumentStateHelper.constructor
        )

        ErrorLogEntry.objects.register(model=WorkflowStateAction)

        WorkflowAction.initialize()

        ModelAttribute(
            Document, 'workflow.< workflow internal name >.get_current_state',
            label=_('Current state of a workflow'), description=_(
                'Return the current state of the selected workflow'
            ), type_name=['property', 'indexing']
        )
        ModelAttribute(
            Document,
            'workflow.< workflow internal name >.get_current_state.completion',
            label=_('Current state of a workflow'), description=_(
                'Return the completion value of the current state of the '
                'selected workflow'
            ), type_name=['property', 'indexing']
        )

        ModelPermission.register(
            model=Workflow, permissions=(
                permission_error_log_view, permission_workflow_delete,
                permission_workflow_edit, permission_workflow_transition,
                permission_workflow_view,
            )
        )

        ModelPermission.register_inheritance(
            model=WorkflowInstance, related='workflow',
        )
        ModelPermission.register_inheritance(
            model=WorkflowInstanceLogEntry,
            related='workflow_instance__workflow',
        )
        ModelPermission.register(
            model=WorkflowTransition,
            permissions=(permission_workflow_transition,)
        )

        ModelPermission.register_inheritance(
            model=WorkflowState, related='workflow',
        )
        ModelPermission.register_inheritance(
            model=WorkflowStateAction, related='state__workflow',
        )
        ModelPermission.register_inheritance(
            model=WorkflowTransition, related='workflow',
        )
        ModelPermission.register_inheritance(
            model=WorkflowTransitionTriggerEvent,
            related='transition__workflow',
        )

        SourceColumn(
            source=Workflow, label=_('Label'), attribute='label'
        )
        SourceColumn(
            source=Workflow, label=_('Internal name'),
            attribute='internal_name'
        )
        SourceColumn(
            source=Workflow, label=_('Initial state'),
            func=lambda context: context['object'].get_initial_state() or _('None')
        )

        SourceColumn(
            source=WorkflowInstance, label=_('Current state'),
            attribute='get_current_state'
        )
        SourceColumn(
            source=WorkflowInstance, label=_('User'),
            func=lambda context: getattr(
                context['object'].get_last_log_entry(), 'user', _('None')
            )
        )
        SourceColumn(
            source=WorkflowInstance, label=_('Last transition'),
            attribute='get_last_transition'
        )
        SourceColumn(
            source=WorkflowInstance, label=_('Date and time'),
            func=lambda context: getattr(
                context['object'].get_last_log_entry(), 'datetime', _('None')
            )
        )
        SourceColumn(
            source=WorkflowInstance, label=_('Completion'),
            func=lambda context: getattr(
                context['object'].get_current_state(), 'completion', _('None')
            )
        )

        SourceColumn(
            source=WorkflowInstanceLogEntry, label=_('Date and time'),
            attribute='datetime'
        )
        SourceColumn(
            source=WorkflowInstanceLogEntry, label=_('User'), attribute='user'
        )
        SourceColumn(
            source=WorkflowInstanceLogEntry, label=_('Transition'),
            attribute='transition'
        )
        SourceColumn(
            source=WorkflowInstanceLogEntry, label=_('Comment'),
            attribute='comment'
        )

        SourceColumn(
            source=WorkflowState, label=_('Is initial state?'),
            func=lambda context: two_state_template(context['object'].initial)
        )
        SourceColumn(
            source=WorkflowState, label=_('Completion'), attribute='completion'
        )

        SourceColumn(
            source=WorkflowStateAction, label=_('Label'), attribute='label'
        )
        SourceColumn(
            source=WorkflowStateAction, label=_('Enabled?'),
            func=lambda context: two_state_template(context['object'].enabled)
        )
        SourceColumn(
            source=WorkflowStateAction, label=_('When?'),
            attribute='get_when_display'
        )
        SourceColumn(
            source=WorkflowStateAction, label=_('Action type'),
            attribute='get_class_label'
        )

        SourceColumn(
            source=WorkflowTransition, label=_('Origin state'),
            attribute='origin_state'
        )
        SourceColumn(
            source=WorkflowTransition, label=_('Destination state'),
            attribute='destination_state'
        )
        SourceColumn(
            source=WorkflowTransition, label=_('Triggers'),
            func=lambda context: widget_transition_events(
                transition=context['object']
            )
        )

        app.conf.CELERY_QUEUES.extend(
            (
                Queue(
                    'document_states', Exchange('document_states'),
                    routing_key='document_states'
                ),
            )
        )

        app.conf.CELERY_ROUTES.update(
            {
                'document_states.tasks.task_launch_all_workflows': {
                    'queue': 'document_states'
                },
            }
        )

        menu_facet.bind_links(
            links=(link_document_workflow_instance_list,), sources=(Document,)
        )
        menu_main.bind_links(links=(link_workflow_list,), position=10)
        menu_object.bind_links(
            links=(
                link_setup_workflow_states, link_setup_workflow_transitions,
                link_setup_workflow_document_types, link_setup_workflow_edit,
                link_acl_list, link_workflow_preview,
                link_setup_workflow_delete
            ), sources=(Workflow,)
        )
        menu_object.bind_links(
            links=(
                link_setup_workflow_state_edit,
                link_setup_workflow_state_action_list,
                link_setup_workflow_state_delete
            ), sources=(WorkflowState,)
        )
        menu_object.bind_links(
            links=(
                link_setup_workflow_transition_edit,
                link_workflow_instance_transition_events, link_acl_list,
                link_setup_workflow_transition_delete
            ), sources=(WorkflowTransition,)
        )
        menu_object.bind_links(
            links=(
                link_workflow_instance_detail,
                link_workflow_instance_transition
            ), sources=(WorkflowInstance,)
        )
        menu_object.bind_links(
            links=(
                link_workflow_document_list, link_workflow_state_list,
            ), sources=(WorkflowRuntimeProxy,)
        )
        menu_object.bind_links(
            links=(
                link_workflow_state_document_list,
            ), sources=(WorkflowStateRuntimeProxy,)
        )
        menu_object.bind_links(
            links=(
                link_setup_workflow_state_action_edit,
                link_object_error_list,
                link_setup_workflow_state_action_delete,
            ), sources=(WorkflowStateAction,)
        )

        menu_secondary.bind_links(
            links=(link_setup_workflow_list, link_setup_workflow_create),
            sources=(
                Workflow, 'document_states:setup_workflow_create',
                'document_states:setup_workflow_list'
            )
        )
        menu_secondary.bind_links(
            links=(link_workflow_list,),
            sources=(
                WorkflowRuntimeProxy,
            )
        )
        menu_secondary.bind_links(
            links=(link_setup_workflow_state_action_selection,),
            sources=(
                WorkflowState,
            )
        )
        menu_setup.bind_links(links=(link_setup_workflow_list,))
        menu_sidebar.bind_links(
            links=(
                link_setup_workflow_state_create,
                link_setup_workflow_transition_create
            ), sources=(Workflow,)
        )
        menu_tools.bind_links(links=(link_tool_launch_all_workflows,))

        post_save.connect(
            launch_workflow, dispatch_uid='launch_workflow', sender=Document
        )

        # Index updating

        post_save.connect(
            handler_index_document,
            dispatch_uid='handler_index_document_save',
            sender=WorkflowInstanceLogEntry
        )
        post_save.connect(
            handler_trigger_transition,
            dispatch_uid='document_states_handler_trigger_transition',
            sender=Action
        )



from __future__ import absolute_import, unicode_literals

from django.contrib import messages
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _, ungettext

from common.generics import (
    FormView, MultipleObjectConfirmActionView, SingleObjectDetailView,
    SingleObjectDownloadView, SingleObjectListView
)
from documents.models import Document

from .forms import DocumentContentForm, DocumentTypeSelectForm
from .models import DocumentVersionParseError
from .permissions import permission_content_view, permission_parse_document
from .utils import get_document_content


class DocumentContentView(SingleObjectDetailView):
    form_class = DocumentContentForm
    model = Document
    object_permission = permission_content_view

    def dispatch(self, request, *args, **kwargs):
        result = super(DocumentContentView, self).dispatch(
            request, *args, **kwargs
        )
        self.get_object().add_as_recent_document_for_user(request.user)
        return result

    def get_extra_context(self):
        return {
            'document': self.get_object(),
            'hide_labels': True,
            'object': self.get_object(),
            'title': _('Content for document: %s') % self.get_object(),
        }


class DocumentContentDownloadView(SingleObjectDownloadView):
    model = Document
    object_permission = permission_content_view

    def get_file(self):
        file_object = DocumentContentDownloadView.TextIteratorIO(
            iterator=get_document_content(document=self.get_object())
        )
        return DocumentContentDownloadView.VirtualFile(
            file=file_object, name='{}-content'.format(self.get_object())
        )


class DocumentParsingErrorsListView(SingleObjectListView):
    view_permission = permission_content_view

    def get_document(self):
        return get_object_or_404(Document, pk=self.kwargs['pk'])

    def get_extra_context(self):
        return {
            'hide_object': True,
            'object': self.get_document(),
            'title': _(
                'Parsing errors for document: %s'
            ) % self.get_document(),
        }

    def get_object_list(self):
        return self.get_document().latest_version.parsing_errors.all()


class DocumentSubmitView(MultipleObjectConfirmActionView):
    model = Document
    object_permission = permission_parse_document
    success_message = _(
        '%(count)d document added to the parsing queue'
    )
    success_message_plural = _(
        '%(count)d documents added to the parsing queue'
    )

    def get_extra_context(self):
        queryset = self.get_queryset()

        result = {
            'title': ungettext(
                singular='Submit %(count)d document to the parsing queue?',
                plural='Submit %(count)d documents to the parsing queue',
                number=queryset.count()
            ) % {
                'count': queryset.count(),
            }
        }

        if queryset.count() == 1:
            result.update(
                {
                    'object': queryset.first(),
                    'title': _(
                        'Submit document "%s" to the parsing queue'
                    ) % queryset.first()
                }
            )

        return result

    def object_action(self, instance, form=None):
        instance.submit_for_parsing()


class DocumentTypeSubmitView(FormView):
    form_class = DocumentTypeSelectForm
    extra_context = {
        'title': _('Submit all documents of a type for parsing')
    }

    def get_form_extra_kwargs(self):
        return {
            'user': self.request.user
        }

    def get_post_action_redirect(self):
        return reverse('common:tools_list')

    def form_valid(self, form):
        count = 0
        for document in form.cleaned_data['document_type'].documents.all():
            document.submit_for_parsing()
            count += 1

        messages.success(
            self.request, _(
                '%(count)d documents of type "%(document_type)s" added to the '
                'parsing queue.'
            ) % {
                'count': count,
                'document_type': form.cleaned_data['document_type']
            }
        )

        return HttpResponseRedirect(self.get_success_url())


class ParseErrorListView(SingleObjectListView):
    extra_context = {
        'hide_object': True,
        'title': _('Parsing errors'),
    }
    view_permission = permission_content_view

    def get_object_list(self):
        return DocumentVersionParseError.objects.all()



from __future__ import absolute_import, unicode_literals

from django.apps import apps
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from common.classes import DashboardWidget

from .statistics import (
    new_document_pages_this_month, new_documents_this_month,
)


def get_total_documents_queryset():
    Document = apps.get_model(
        app_label='documents', model_name='Document'
    )
    return Document.objects.all()


def get_document_types_queryset():
    DocumentType = apps.get_model(
        app_label='documents', model_name='DocumentType'
    )
    return DocumentType.objects.all()


def get_deleted_documents_queryset():
    DeletedDocument = apps.get_model(
        app_label='documents', model_name='DeletedDocument'
    )
    return DeletedDocument.objects.all()


widget_pages_per_month = DashboardWidget(
    func=new_document_pages_this_month, icon='fa fa-calendar',
    label=_('New pages this month'),
    link=reverse_lazy(
        'statistics:statistic_detail',
        args=('new-document-pages-per-month',)
    )
)

widget_new_documents_this_month = DashboardWidget(
    func=new_documents_this_month, icon='fa fa-calendar',
    label=_('New documents this month'),
    link=reverse_lazy(
        'statistics:statistic_detail',
        args=('new-documents-per-month',)
    )
)

widget_total_documents = DashboardWidget(
    icon='fa fa-file', queryset=get_total_documents_queryset,
    label=_('Total documents'),
    link=reverse_lazy('documents:document_list')
)


widget_document_types = DashboardWidget(
    icon='fa fa-book', queryset=get_document_types_queryset,
    label=_('Document types'),
    link=reverse_lazy('documents:document_type_list')
)


widget_documents_in_trash = DashboardWidget(
    icon='fa fa-trash', queryset=get_deleted_documents_queryset,
    label=_('Documents in trash'),
    link=reverse_lazy('documents:document_list_deleted')
)



from __future__ import unicode_literals

import logging

from django.utils.module_loading import import_string

from .settings import setting_graphics_backend

logger = logging.getLogger(__name__)
backend = converter_class = import_string(setting_graphics_backend.value)



from __future__ import unicode_literals

from django.conf.urls import url

from .api_views import APIDocumentPageContentView
from .views import (
    DocumentContentView, DocumentContentDownloadView,
    DocumentParsingErrorsListView, DocumentSubmitView, DocumentTypeSubmitView,
    ParseErrorListView
)

urlpatterns = [
    url(
        r'^documents/(?P<pk>\d+)/content/$', DocumentContentView.as_view(),
        name='document_content'
    ),
    url(
        r'^documents/(?P<pk>\d+)/content/download/$',
        DocumentContentDownloadView.as_view(), name='document_content_download'
    ),
    url(
        r'^document_types/submit/$', DocumentTypeSubmitView.as_view(),
        name='document_type_submit'
    ),
    url(
        r'^documents/(?P<pk>\d+)/submit/$', DocumentSubmitView.as_view(),
        name='document_submit'
    ),
    url(
        r'^documents/multiple/submit/$', DocumentSubmitView.as_view(),
        name='document_submit_multiple'
    ),
    url(
        r'^documents/(?P<pk>\d+)/errors/$',
        DocumentParsingErrorsListView.as_view(),
        name='document_parsing_error_list'
    ),
    url(r'^errors/all/$', ParseErrorListView.as_view(), name='error_list'),
]

api_urls = [
    url(
        r'^page/(?P<pk>\d+)/content/$', APIDocumentPageContentView.as_view(),
        name='document-page-content-view'
    ),
]



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from common.queues import queue_tools
from task_manager.classes import CeleryQueue

queue_signatures = CeleryQueue(name='signatures', label=_('Signatures'))
queue_signatures.add_task_type(
    name='document_signatures.tasks.task_verify_key_signatures',
    label=_('Verify key signatures')
)
queue_signatures.add_task_type(
    name='document_signatures.tasks.task_unverify_key_signatures',
    label=_('Unverify key signatures')
)
queue_signatures.add_task_type(
    name='document_signatures.tasks.task_verify_document_version',
    label=_('Verify document version')
)

queue_tools.add_task_type(
    name='document_signatures.tasks.task_verify_missing_embedded_signature',
    label=_('Verify missing embedded signature')
)



from __future__ import unicode_literals

import logging
import sys
import traceback

from django.conf import settings
from django.db import models

from .events import event_parsing_document_version_finish
from .parsers import Parser

logger = logging.getLogger(__name__)


class DocumentPageContentManager(models.Manager):
    def process_document_version(self, document_version):
        logger.info(
            'Starting parsing for document version: %s', document_version
        )
        logger.debug('document version: %d', document_version.pk)

        try:
            Parser.parse_document_version(document_version=document_version)
        except Exception as exception:
            logger.error(
                'Parsing error for document version: %d; %s',
                document_version.pk, exception,
            )

            if settings.DEBUG:
                result = []
                type, value, tb = sys.exc_info()
                result.append('%s: %s' % (type.__name__, value))
                result.extend(traceback.format_tb(tb))
                document_version.parsing_errors.create(
                    result='\n'.join(result)
                )
            else:
                document_version.parsing_errors.create(result=exception)
        else:
            logger.info(
                'Parsing complete for document version: %s', document_version
            )
            document_version.parsing_errors.all().delete()

            event_parsing_document_version_finish.commit(
                action_object=document_version.document,
                target=document_version
            )



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-25 11:11
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0039_duplicateddocument'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='documentversion',
            options={'ordering': ('timestamp',), 'verbose_name': 'Document version', 'verbose_name_plural': 'Document version'},
        ),
    ]



from __future__ import unicode_literals

from importlib import import_module
import logging
import os

import yaml

from django.apps import apps
from django.conf import settings
from django.utils.functional import Promise
from django.utils.encoding import force_text, python_2_unicode_compatible

logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class Namespace(object):
    _registry = {}

    @staticmethod
    def initialize():
        for app in apps.get_app_configs():
            try:
                import_module('{}.settings'.format(app.name))
            except ImportError as exception:
                if force_text(exception) not in ('No module named settings', 'No module named \'{}.settings\''.format(app.name)):
                    logger.error(
                        'Error importing %s settings.py file; %s', app.name,
                        exception
                    )

    @classmethod
    def get_all(cls):
        return sorted(cls._registry.values(), key=lambda x: x.label)

    @classmethod
    def get(cls, name):
        return cls._registry[name]

    @classmethod
    def invalidate_cache_all(cls):
        for namespace in cls.get_all():
            namespace.invalidate_cache()

    def __str__(self):
        return force_text(self.label)

    def __init__(self, name, label):
        if name in self.__class__._registry:
            raise Exception(
                'Namespace names must be unique; "%s" already exists.' % name
            )
        self.name = name
        self.label = label
        self.__class__._registry[name] = self
        self._settings = []

    def add_setting(self, **kwargs):
        return Setting(namespace=self, **kwargs)

    def invalidate_cache(self):
        for setting in self._settings:
            setting.invalidate_cache()

    @property
    def settings(self):
        return sorted(self._settings, key=lambda x: x.global_name)


@python_2_unicode_compatible
class Setting(object):
    _registry = {}

    @staticmethod
    def deserialize_value(value):
        return yaml.safe_load(value)

    @staticmethod
    def serialize_value(value):
        if isinstance(value, Promise):
            value = force_text(value)

        result = yaml.safe_dump(value, allow_unicode=True)
        # safe_dump returns bytestrings
        # Disregard the last 3 dots that mark the end of the YAML document
        if force_text(result).endswith('...\n'):
            result = result[:-4]

        return result

    @classmethod
    def get(cls, global_name):
        return cls._registry[global_name].value

    def __init__(self, namespace, global_name, default, help_text=None, is_path=False):
        self.global_name = global_name
        self.default = default
        self.help_text = help_text
        self.loaded = False
        namespace._settings.append(self)
        self.__class__._registry[global_name] = self

    def __str__(self):
        return force_text(self.global_name)

    def cache_value(self):
        environment_value = os.environ.get('MAYAN_{}'.format(self.global_name))
        if environment_value:
            self.raw_value = yaml.safe_load(environment_value)
        else:
            self.raw_value = getattr(settings, self.global_name, self.default)
        self.yaml = Setting.serialize_value(self.raw_value)
        self.loaded = True

    def invalidate_cache(self):
        self.loaded = False

    @property
    def serialized_value(self):
        """
        YAML serialize value of the setting.
        Used for UI display.
        """
        if not self.loaded:
            self.cache_value()

        return self.yaml

    @property
    def value(self):
        if not self.loaded:
            self.cache_value()

        return self.raw_value

    @value.setter
    def value(self, value):
        # value is in YAML format
        self.yaml = value
        self.raw_value = Setting.deserialize_value(value)



from __future__ import unicode_literals

from actstream.models import Action

from documents.tests.literals import TEST_DOCUMENT_FILENAME
from documents.tests.test_models import GenericDocumentTestCase

from ..events import (
    event_parsing_document_version_submit,
    event_parsing_document_version_finish
)


class DocumentParsingEventsTestCase(GenericDocumentTestCase):
    # Ensure we use a PDF file
    test_document_filename = TEST_DOCUMENT_FILENAME

    def test_document_version_submit_event(self):
        Action.objects.all().delete()
        self.document.submit_for_parsing()

        self.assertEqual(
            Action.objects.last().target, self.document.latest_version
        )
        self.assertEqual(
            Action.objects.last().verb,
            event_parsing_document_version_submit.name
        )

    def test_document_version_finish_event(self):
        Action.objects.all().delete()
        self.document.submit_for_parsing()
        self.assertEqual(
            Action.objects.first().target, self.document.latest_version
        )
        self.assertEqual(
            Action.objects.first().verb,
            event_parsing_document_version_finish.name
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-05-30 07:28
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('document_indexing', '0011_auto_20170524_0456'),
    ]

    operations = [
        migrations.AlterField(
            model_name='index',
            name='slug',
            field=models.SlugField(help_text='This value will be used by other apps to reference this index.', max_length=128, unique=True, verbose_name='Slug'),
        ),
        migrations.AlterField(
            model_name='indexinstancenode',
            name='documents',
            field=models.ManyToManyField(related_name='index_instance_nodes', to='documents.Document', verbose_name='Documents'),
        ),
        migrations.AlterField(
            model_name='indexinstancenode',
            name='index_template_node',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='index_instance_nodes', to='document_indexing.IndexTemplateNode', verbose_name='Index template node'),
        ),
    ]



from __future__ import unicode_literals

from actstream.models import Action

from documents.tests.test_models import GenericDocumentTestCase

from ..events import (
    event_cabinets_add_document, event_cabinets_remove_document
)
from ..models import Cabinet

from .literals import TEST_CABINET_LABEL


class CabinetsEventsTestCase(GenericDocumentTestCase):
    def setUp(self):
        super(CabinetsEventsTestCase, self).setUp()
        self._create_cabinet()

    def _create_cabinet(self):
        self.cabinet = Cabinet.objects.create(label=TEST_CABINET_LABEL)

    def test_document_cabinet_add_event(self):
        Action.objects.all().delete()
        self.cabinet.add_document(document=self.document)

        self.assertEqual(Action.objects.last().target, self.document)
        self.assertEqual(
            Action.objects.last().verb,
            event_cabinets_add_document.name
        )

    def test_document_cabinet_remove_event(self):
        self.cabinet.add_document(document=self.document)
        Action.objects.all().delete()
        self.cabinet.remove_document(document=self.document)

        self.assertEqual(Action.objects.first().target, self.document)
        self.assertEqual(
            Action.objects.first().verb,
            event_cabinets_remove_document.name
        )



from __future__ import unicode_literals

from django.apps import apps
from django.utils.translation import ugettext_lazy as _

from document_indexing.tasks import task_index_document
from events.classes import Event


def handler_index_document(sender, **kwargs):
    task_index_document.apply_async(
        kwargs=dict(
            document_id=kwargs['instance'].workflow_instance.document.pk
        )
    )


def handler_trigger_transition(sender, **kwargs):
    action = kwargs['instance']

    Document = apps.get_model(
        app_label='documents', model_name='Document'
    )
    WorkflowInstance = apps.get_model(
        app_label='document_states', model_name='WorkflowInstance'
    )
    WorkflowTransition = apps.get_model(
        app_label='document_states', model_name='WorkflowTransition'
    )

    trigger_transitions = WorkflowTransition.objects.filter(trigger_events__event_type__name=kwargs['instance'].verb)

    if isinstance(action.target, Document):
        workflow_instances = WorkflowInstance.objects.filter(workflow__transitions__in=trigger_transitions, document=action.target).distinct()
    elif isinstance(action.action_object, Document):
        workflow_instances = WorkflowInstance.objects.filter(workflow__transitions__in=trigger_transitions, document=action.action_object).distinct()
    else:
        workflow_instances = WorkflowInstance.objects.none()

    for workflow_instance in workflow_instances:
        # Select the first transition that is valid for this workflow state
        transition = list(set(trigger_transitions) & set(workflow_instance.get_transition_choices()))[0]

        workflow_instance.do_transition(
            comment=_('Event trigger: %s') % Event.get(name=action.verb).label,
            transition=transition
        )


def launch_workflow(sender, instance, created, **kwargs):
    Workflow = apps.get_model(
        app_label='document_states', model_name='Workflow'
    )

    if created:
        Workflow.objects.launch_for(instance)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-01-18 17:58
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tags', '0006_documenttag'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='tag',
            options={'ordering': ('label',), 'verbose_name': 'Tag', 'verbose_name_plural': 'Tags'},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-03 07:28
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('document_states', '0006_auto_20170803_0651'),
    ]

    operations = [
        migrations.AlterField(
            model_name='workflowinstancelogentry',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AlterField(
            model_name='workflowtransitiontriggerevent',
            name='event_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='events.EventType', verbose_name='Event type'),
        ),
        migrations.AlterField(
            model_name='workflowtransitiontriggerevent',
            name='transition',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='trigger_events', to='document_states.WorkflowTransition', verbose_name='Transition'),
        ),
    ]



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from permissions import PermissionNamespace

namespace = PermissionNamespace('document_parsing', _('Document parsing'))

permission_content_view = namespace.add_permission(
    name='content_view', label=_('View the content of a document')
)

permission_parse_document = namespace.add_permission(
    name='parse_document', label=_('Parse the content of a document')
)



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from task_manager.classes import CeleryQueue

queue_checkouts_periodic = CeleryQueue(
    name='checkouts_periodic', label=_('Checkouts periodic'), transient=True
)
queue_checkouts_periodic.add_task_type(
    name='task_check_expired_check_outs',
    label=_('Check expired checkouts')
)



from __future__ import unicode_literals

from django.urls import reverse
from django.utils.safestring import mark_safe

from .classes import Event


def event_object_link(entry, attribute='target'):
    obj = getattr(entry, attribute)

    if obj:
        obj_type = '{}: '.format(obj._meta.verbose_name)
    else:
        obj_type = ''

    return mark_safe(
        '<a href="%(url)s">%(obj_type)s%(label)s</a>' % {
            'url': obj.get_absolute_url() if obj else '#',
            'label': obj or '', 'obj_type': obj_type
        }
    )


def event_type_link(entry):
    return mark_safe(
        '<a href="%(url)s">%(label)s</a>' % {
            'url': reverse('events:events_by_verb', kwargs={'verb': entry.verb}),
            'label': Event.get_label(entry.verb)
        }
    )



from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from documents.models import DocumentPage, DocumentType, DocumentVersion

from .managers import DocumentPageOCRContentManager


class DocumentTypeSettings(models.Model):
    """
    Define for OCR for a specific document should behave
    """
    document_type = models.OneToOneField(
        DocumentType, on_delete=models.CASCADE, related_name='ocr_settings',
        unique=True, verbose_name=_('Document type')
    )
    auto_ocr = models.BooleanField(
        default=True,
        verbose_name=_('Automatically queue newly created documents for OCR.')
    )

    class Meta:
        verbose_name = _('Document type settings')
        verbose_name_plural = _('Document types settings')


@python_2_unicode_compatible
class DocumentPageOCRContent(models.Model):
    document_page = models.OneToOneField(
        DocumentPage, on_delete=models.CASCADE, related_name='ocr_content',
        verbose_name=_('Document page')
    )
    content = models.TextField(blank=True, verbose_name=_('Content'))

    objects = DocumentPageOCRContentManager()

    def __str__(self):
        return force_text(self.document_page)

    class Meta:
        verbose_name = _('Document page OCR content')
        verbose_name_plural = _('Document pages OCR contents')


@python_2_unicode_compatible
class DocumentVersionOCRError(models.Model):
    document_version = models.ForeignKey(
        DocumentVersion, on_delete=models.CASCADE, related_name='ocr_errors',
        verbose_name=_('Document version')
    )
    datetime_submitted = models.DateTimeField(
        auto_now_add=True, db_index=True, verbose_name=_('Date time submitted')
    )
    result = models.TextField(blank=True, null=True, verbose_name=_('Result'))

    def __str__(self):
        return force_text(self.document_version)

    class Meta:
        ordering = ('datetime_submitted',)
        verbose_name = _('Document version OCR error')
        verbose_name_plural = _('Document version OCR errors')



from __future__ import unicode_literals

from django.core import management

from ...signals import post_initial_setup, pre_initial_setup


class Command(management.BaseCommand):
    help = 'Initializes an install and gets it ready to be used.'

    def handle(self, *args, **options):
        management.call_command('createsettings', interactive=False)
        pre_initial_setup.send(sender=self)
        management.call_command('createautoadmin', interactive=False)
        post_initial_setup.send(sender=self)



from __future__ import absolute_import, unicode_literals

import logging

from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from common.views import (
    SingleObjectCreateView, SingleObjectDeleteView, SingleObjectEditView,
    SingleObjectListView
)

from .models import Message
from .permissions import (
    permission_message_create, permission_message_delete,
    permission_message_edit, permission_message_view
)

logger = logging.getLogger(__name__)


class MessageCreateView(SingleObjectCreateView):
    fields = ('label', 'message', 'enabled', 'start_datetime', 'end_datetime')
    model = Message
    view_permission = permission_message_create

    def get_extra_context(self):
        return {
            'title': _('Create message'),
        }


class MessageDeleteView(SingleObjectDeleteView):
    model = Message
    object_permission = permission_message_delete
    post_action_redirect = reverse_lazy('motd:message_list')

    def get_extra_context(self):
        return {
            'message': None,
            'object': self.get_object(),
            'title': _('Delete the message: %s?') % self.get_object(),
        }


class MessageEditView(SingleObjectEditView):
    fields = ('label', 'message', 'enabled', 'start_datetime', 'end_datetime')
    model = Message
    object_permission = permission_message_edit
    post_action_redirect = reverse_lazy('motd:message_list')

    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'title': _('Edit message: %s') % self.get_object(),
        }


class MessageListView(SingleObjectListView):
    model = Message
    object_permission = permission_message_view

    def get_extra_context(self):
        return {
            'hide_link': True,
            'title': _('Messages'),
        }



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from events.classes import Event

event_tag_attach = Event(
    name='tag_attach',
    label=_('Tag attached to document')
)
event_tag_remove = Event(
    name='tag_remove',
    label=_('Tag removed from document')
)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from task_manager.classes import CeleryQueue

queue_ocr = CeleryQueue(name='parsing', label=_('Parsing'))
queue_ocr.add_task_type(
    name='document_parsing.tasks.task_parse_document_version',
    label=_('Document version parsing')
)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-25 06:52
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('common', '0007_auto_20170118_1758'),
    ]

    operations = [
        migrations.CreateModel(
            name='ErrorLogEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('namespace', models.CharField(max_length=128, verbose_name='Namespace')),
                ('object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('datetime', models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='Date time')),
                ('result', models.TextField(blank=True, null=True, verbose_name='Result')),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='error_log_content_type', to='contenttypes.ContentType')),
            ],
            options={
                'ordering': ('datetime',),
                'verbose_name': 'Error log entry',
                'verbose_name_plural': 'Error log entries',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-07 06:12
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('document_states', '0008_auto_20170803_0752'),
    ]

    operations = [
        migrations.CreateModel(
            name='WorkflowStateAction',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(max_length=255, verbose_name='Label')),
                ('enabled', models.BooleanField(default=True, verbose_name='Enabled')),
                ('when', models.PositiveIntegerField(choices=[(1, 'On entry'), (2, 'On exit')], default=1, help_text='At which moment of the state this action will execute', verbose_name='When')),
                ('action_path', models.CharField(help_text='The dotted Python path to the workflow action class to execute.', max_length=128, verbose_name='Entry action path')),
                ('action_data', models.TextField(blank=True, verbose_name='Entry action data')),
                ('state', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='actions', to='document_states.WorkflowState', verbose_name='Workflow state')),
            ],
            options={
                'ordering': ('label',),
                'verbose_name': 'Workflow state action',
                'verbose_name_plural': 'Workflow state actions',
            },
        ),
        migrations.AlterUniqueTogether(
            name='workflowstateaction',
            unique_together=set([('state', 'label')]),
        ),
    ]



from __future__ import unicode_literals

from django.core.files.base import File
from django.test import override_settings

from common.tests import BaseTestCase
from documents.models import DocumentType
from documents.tests import TEST_DOCUMENT_PATH, TEST_DOCUMENT_TYPE_LABEL

from ..parsers import PopplerParser


@override_settings(OCR_AUTO_OCR=False)
class ParserTestCase(BaseTestCase):
    def setUp(self):
        super(ParserTestCase, self).setUp()
        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE_LABEL
        )

        with open(TEST_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=File(file_object)
            )

    def tearDown(self):
        self.document_type.delete()
        super(ParserTestCase, self).tearDown()

    def test_poppler_parser(self):
        parser = PopplerParser()

        parser.process_document_version(self.document.latest_version)

        self.assertTrue(
            'Mayan EDMS Documentation' in self.document.pages.first().content.content
        )



from __future__ import unicode_literals

from django.test import override_settings

from documents.tests.test_views import GenericDocumentViewTestCase

from ..permissions import permission_ocr_content_view
from ..utils import get_document_ocr_content


@override_settings(OCR_AUTO_OCR=True)
class OCRViewsTestCase(GenericDocumentViewTestCase):
    # PyOCR's leak descriptor in get_available_languages and image_to_string
    # Disable descriptor leak test until fixed in upstream
    _skip_file_descriptor_test = True

    def setUp(self):
        super(OCRViewsTestCase, self).setUp()
        self.login_user()

    def _document_content_view(self):
        return self.get(
            'ocr:document_content', args=(self.document.pk,)
        )

    def test_document_content_view_no_permissions(self):
        response = self._document_content_view()

        self.assertEqual(response.status_code, 403)

    def test_document_content_view_with_permission(self):
        self.grant_permission(permission=permission_ocr_content_view)

        response = self._document_content_view()

        self.assertContains(
            response, 'Mayan EDMS Documentation', status_code=200
        )

    def test_document_ocr_download_view_no_permission(self):
        response = self.get(
            'ocr:document_ocr_download', args=(self.document.pk,)
        )

        self.assertEqual(response.status_code, 403)

    def test_document_download_view_with_permission(self):
        self.expected_content_type = 'application/octet-stream; charset=utf-8'

        self.grant_permission(permission=permission_ocr_content_view)
        response = self.get(
            'ocr:document_ocr_download', args=(self.document.pk,)
        )

        self.assertEqual(response.status_code, 200)

        self.assert_download_response(
            response, content=(
                ''.join(get_document_ocr_content(document=self.document))
            ),
        )



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from permissions import PermissionNamespace

namespace = PermissionNamespace('mailing', _('Mailing'))

permission_mailing_link = namespace.add_permission(
    name='mail_link', label=_('Send document link via email')
)
permission_mailing_send_document = namespace.add_permission(
    name='mail_document', label=_('Send document via email')
)
permission_view_error_log = namespace.add_permission(
    name='view_error_log', label=_('View system mailing error log')
)
permission_user_mailer_create = namespace.add_permission(
    name='user_mailer_create', label=_('Create a mailing profile')
)
permission_user_mailer_delete = namespace.add_permission(
    name='user_mailer_delete', label=_('Delete a mailing profile')
)
permission_user_mailer_edit = namespace.add_permission(
    name='user_mailer_edit', label=_('Edit a mailing profile')
)
permission_user_mailer_view = namespace.add_permission(
    name='user_mailer_view', label=_('View a mailing profile')
)
permission_user_mailer_use = namespace.add_permission(
    name='user_mailer_use', label=_('Use a mailing profile')
)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from common import (
    MayanAppConfig, menu_object, menu_secondary, menu_tools
)
from common.widgets import two_state_template
from navigation import SourceColumn

from .classes import CeleryQueue, Task
from .links import (
    link_queue_list, link_queue_active_task_list,
    link_queue_scheduled_task_list, link_queue_reserved_task_list,
    link_task_manager
)


class TaskManagerApp(MayanAppConfig):
    app_namespace = 'task_manager'
    app_url = 'task_manager'
    name = 'task_manager'
    verbose_name = _('Task manager')

    def ready(self):
        super(TaskManagerApp, self).ready()

        SourceColumn(
            source=CeleryQueue, label=_('Label'), attribute='label'
        )
        SourceColumn(
            source=CeleryQueue, label=_('Name'), attribute='name'
        )
        SourceColumn(
            source=CeleryQueue, label=_('Default queue?'),
            func=lambda context: two_state_template(
                context['object'].is_default_queue
            )
        )
        SourceColumn(
            source=CeleryQueue, label=_('Is transient?'),
            func=lambda context: two_state_template(
                context['object'].is_transient
            )
        )
        SourceColumn(
            source=Task, label=_('Type'), attribute='task_type'
        )
        SourceColumn(
            source=Task, label=_('Start time'), attribute='get_time_started'
        )
        SourceColumn(
            source=Task, label=_('Host'),
            func=lambda context: context['object'].kwargs['hostname']
        )
        SourceColumn(
            source=Task, label=_('Acknowledged'),
            func=lambda context: two_state_template(
                context['object'].kwargs['acknowledged']
            )
        )
        SourceColumn(
            source=Task, label=_('Arguments'),
            func=lambda context: context['object'].kwargs['args']
        )
        SourceColumn(
            source=Task, label=_('Keyword arguments'),
            func=lambda context: context['object'].kwargs['kwargs']
        )
        SourceColumn(
            source=Task, label=_('Worker process ID'),
            func=lambda context: context['object'].kwargs['worker_pid']
        )

        menu_object.bind_links(
            links=(
                link_queue_active_task_list, link_queue_scheduled_task_list,
                link_queue_reserved_task_list,
            ), sources=(CeleryQueue,)
        )

        menu_secondary.bind_links(
            links=(link_queue_list,),
            sources=(CeleryQueue, Task, 'task_manager:queue_list')
        )

        menu_tools.bind_links(links=(link_task_manager,))



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from events.classes import Event

event_parsing_document_version_submit = Event(
    name='parsing_document_version_submit',
    label=_('Document version submitted for parsing')
)
event_parsing_document_version_finish = Event(
    name='parsing_document_version_finish',
    label=_('Document version parsing finished')
)



from __future__ import unicode_literals

import logging
import sys
import traceback

from django.apps import apps
from django.conf import settings
from django.db import models

from documents.runtime import cache_storage_backend

from .events import event_ocr_document_version_finish
from .runtime import ocr_backend
from .signals import post_document_version_ocr

logger = logging.getLogger(__name__)


class DocumentPageOCRContentManager(models.Manager):
    def process_document_version(self, document_version):
        logger.info('Starting OCR for document version: %s', document_version)
        logger.debug('document version: %d', document_version.pk)

        try:
            for document_page in document_version.pages.all():
                self.process_document_page(document_page=document_page)
        except Exception as exception:
            logger.error(
                'OCR error for document version: %d; %s', document_version,
                exception
            )

            if settings.DEBUG:
                result = []
                type, value, tb = sys.exc_info()
                result.append('%s: %s' % (type.__name__, value))
                result.extend(traceback.format_tb(tb))
                document_version.ocr_errors.create(
                    result='\n'.join(result)
                )
            else:
                document_version.ocr_errors.create(result=exception)
        else:
            logger.info(
                'OCR complete for document version: %s', document_version
            )
            document_version.ocr_errors.all().delete()

            event_ocr_document_version_finish.commit(
                action_object=document_version.document,
                target=document_version
            )

            post_document_version_ocr.send(
                sender=document_version.__class__, instance=document_version
            )

    def process_document_page(self, document_page):
        logger.info(
            'Processing page: %d of document version: %s',
            document_page.page_number, document_page.document_version
        )

        DocumentPageOCRContent = apps.get_model(
            app_label='ocr', model_name='DocumentPageOCRContent'
        )

        # TODO: Call task and wait
        cache_filename = document_page.generate_image()

        with cache_storage_backend.open(cache_filename) as file_object:
            document_page_content, created = DocumentPageOCRContent.objects.get_or_create(
                document_page=document_page
            )
            document_page_content.content = ocr_backend.execute(
                file_object=file_object,
                language=document_page.document.language
            )
            document_page_content.save()

        logger.info(
            'Finished processing page: %d of document version: %s',
            document_page.page_number, document_page.document_version
        )



from __future__ import unicode_literals

from django import forms
from django.urls import reverse
from django.utils.html import format_html_join, mark_safe


def widget_transition_events(transition):
    return format_html_join(
        sep='\n', format_string='<div class="">{}</div>', args_generator=(
            (
                transition_trigger.event_type.label,
            ) for transition_trigger in transition.trigger_events.all()
        )
    )


def widget_workflow_diagram(workflow):
    return mark_safe(
        '<img class="img-responsive" src="{}">'.format(
            reverse('document_states:workflow_image', args=(workflow.pk,))
        )
    )


class WorkflowImageWidget(forms.widgets.Widget):
    def render(self, name, value, attrs=None):
        final_attrs = self.build_attrs(attrs)

        if value:
            output = []
            output.append(widget_workflow_diagram(value))
            return mark_safe(''.join(output))
        else:
            return ''



from __future__ import unicode_literals

from django.conf import settings
from django.conf.urls import url
from django.contrib.auth.views import logout

from .views import (
    login_view, password_change_done, password_change_view,
    password_reset_complete_view, password_reset_confirm_view,
    password_reset_done_view, password_reset_view
)


urlpatterns = [
    url(r'^login/$', login_view, name='login_view'),
    url(
        r'^password/change/done/$', password_change_done,
        name='password_change_done'
    ),
    url(
        r'^password/change/$', password_change_view,
        name='password_change_view'
    ),
    url(
        r'^logout/$', logout, {'next_page': settings.LOGIN_REDIRECT_URL},
        name='logout_view'
    ),
    url(
        r'^password/reset/$', password_reset_view, name='password_reset_view'
    ),
    url(
        r'^password/reset/confirm/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
        password_reset_confirm_view, name='password_reset_confirm_view'
    ),
    url(
        r'^password/reset/complete/$', password_reset_complete_view,
        name='password_reset_complete_view'
    ),
    url(
        r'^password/reset/done/$', password_reset_done_view,
        name='password_reset_done_view'
    ),
]



from __future__ import unicode_literals

import os
import uuid

from django.core import management
from django.conf import settings
from django.utils.encoding import force_text

from ..utils import fs_cleanup
from ..management.commands.literals import SETTING_FILE_TEMPLATE

from .base import BaseTestCase


class CommonCommandsTestCase(BaseTestCase):
    def test_createsettings_command(self):
        filename = force_text(uuid.uuid4())
        with self.settings(COMMON_LOCAL_SETTINGS_FILENAME=filename):
            management.call_command('createsettings', interactive=False)
            file_path = os.path.join(
                settings.BASE_DIR, 'settings', '{}.py'.format(filename)
            )

            with open(file_path) as file_object:
                content = file_object.read()

        fs_cleanup(filename=file_path)
        # Compare without the string substitution and the final linefeeds
        self.assertTrue(
            SETTING_FILE_TEMPLATE.replace("{0}'", '')[0:-2] in content
        )



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from task_manager.classes import CeleryQueue

queue_document_states = CeleryQueue(
    name='document_states', label=_('Document states')
)
queue_document_states.add_task_type(
    name='document_states.tasks.task_launch_all_workflows',
    label=_('Launch all workflows')
)



from __future__ import absolute_import, unicode_literals

from django.apps import apps
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from common.classes import DashboardWidget


def checkedout_documents_queryset():
    DocumentCheckout = apps.get_model(
        app_label='checkouts', model_name='DocumentCheckout'
    )
    return DocumentCheckout.objects.all()


widget_checkouts = DashboardWidget(
    label=_('Checkedout documents'),
    link=reverse_lazy('checkouts:checkout_list'),
    icon='fa fa-shopping-cart', queryset=checkedout_documents_queryset
)



from __future__ import unicode_literals

from actstream.models import Action

from documents.tests.test_models import GenericDocumentTestCase

from ..events import (
    event_ocr_document_version_submit, event_ocr_document_version_finish
)


class OCREventsTestCase(GenericDocumentTestCase):
    def test_document_version_submit_event(self):
        Action.objects.all().delete()
        self.document.submit_for_ocr()

        self.assertEqual(
            Action.objects.last().target, self.document.latest_version
        )
        self.assertEqual(
            Action.objects.last().verb,
            event_ocr_document_version_submit.name
        )

    def test_document_version_finish_event(self):
        Action.objects.all().delete()
        self.document.submit_for_ocr()

        self.assertEqual(
            Action.objects.first().target, self.document.latest_version
        )
        self.assertEqual(
            Action.objects.first().verb,
            event_ocr_document_version_finish.name
        )



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from task_manager.classes import CeleryQueue

queue_ocr = CeleryQueue(name='ocr', label=_('OCR'))
queue_ocr.add_task_type(
    name='ocr.tasks.task_do_ocr', label=_('Document version OCR')
)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-18 01:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mailer', '0004_auto_20170714_2133'),
    ]

    operations = [
        migrations.AlterField(
            model_name='usermailer',
            name='label',
            field=models.CharField(max_length=128, unique=True, verbose_name='Label'),
        ),
    ]



from __future__ import unicode_literals

from django.conf.urls import url

from .views import (
    SystemMailerLogEntryListView, MailDocumentLinkView, MailDocumentView,
    UserMailerBackendSelectionView, UserMailingCreateView,
    UserMailingDeleteView, UserMailingEditView, UserMailerLogEntryListView,
    UserMailerTestView, UserMailerListView
)

urlpatterns = [
    url(
        r'^(?P<pk>\d+)/send/link/$', MailDocumentLinkView.as_view(),
        name='send_document_link'
    ),
    url(
        r'^multiple/send/link/$', MailDocumentLinkView.as_view(),
        name='send_multiple_document_link'
    ),
    url(
        r'^(?P<pk>\d+)/send/document/$', MailDocumentView.as_view(),
        name='send_document'
    ),
    url(
        r'^multiple/send/document/$', MailDocumentView.as_view(),
        name='send_multiple_document'
    ),
    url(
        r'^system_mailer/log/$', SystemMailerLogEntryListView.as_view(),
        name='system_mailer_error_log'
    ),
    url(
        r'^user_mailers/backend/selection/$',
        UserMailerBackendSelectionView.as_view(),
        name='user_mailer_backend_selection'
    ),
    url(
        r'^user_mailers/(?P<class_path>[a-zA-Z0-9_.]+)/create/$',
        UserMailingCreateView.as_view(), name='user_mailer_create'
    ),
    url(
        r'^user_mailers/(?P<pk>\d+)/delete/$', UserMailingDeleteView.as_view(),
        name='user_mailer_delete'
    ),
    url(
        r'^user_mailers/(?P<pk>\d+)/edit/$', UserMailingEditView.as_view(),
        name='user_mailer_edit'
    ),
    url(
        r'^user_mailers/(?P<pk>\d+)/log/$',
        UserMailerLogEntryListView.as_view(), name='user_mailer_log'
    ),
    url(
        r'^user_mailers/(?P<pk>\d+)/test/$',
        UserMailerTestView.as_view(), name='user_mailer_test'
    ),
    url(
        r'^user_mailers/$', UserMailerListView.as_view(),
        name='user_mailer_list'
    ),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-03 06:38
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('events', '0001_initial'),
        ('document_states', '0004_workflow_internal_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='WorkflowTransitionTriggerEvent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('stored_event_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='trigger_events', to='events.EventType', verbose_name='Event type')),
            ],
            options={
                'verbose_name': 'Workflow transition trigger event',
                'verbose_name_plural': 'Workflow transitions trigger events',
            },
        ),
        migrations.AddField(
            model_name='workflowtransition',
            name='trigger_time_period',
            field=models.PositiveIntegerField(blank=True, help_text='Amount of time after which this transition will trigger on its own.', null=True, verbose_name='Trigger time period'),
        ),
        migrations.AddField(
            model_name='workflowtransition',
            name='trigger_time_unit',
            field=models.CharField(blank=True, choices=[('days', 'Days'), ('hours', 'Hours'), ('minutes', 'Minutes')], max_length=8, null=True, verbose_name='Trigger time unit'),
        ),
        migrations.AddField(
            model_name='workflowtransitiontriggerevent',
            name='transition',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='document_states.WorkflowTransition', verbose_name='Transition'),
        ),
    ]



from __future__ import unicode_literals

from document_states.tests.test_actions import ActionTestCase

from ..models import Tag
from ..workflow_actions import AttachTagAction, RemoveTagAction

from .literals import TEST_TAG_COLOR, TEST_TAG_LABEL


class TagActionTestCase(ActionTestCase):
    def setUp(self):
        super(TagActionTestCase, self).setUp()
        self.tag = Tag.objects.create(
            color=TEST_TAG_COLOR, label=TEST_TAG_LABEL
        )

    def test_tag_attach_action(self):
        action = AttachTagAction(form_data={'tags': Tag.objects.all()})
        action.execute(context={'entry_log': self.entry_log})

        self.assertEqual(self.tag.documents.count(), 1)
        self.assertEqual(list(self.tag.documents.all()), [self.document])

    def test_tag_remove_action(self):
        self.tag.attach_to(document=self.document)

        action = RemoveTagAction(form_data={'tags': Tag.objects.all()})
        action.execute(context={'entry_log': self.entry_log})

        self.assertEqual(self.tag.documents.count(), 0)



from __future__ import unicode_literals

from datetime import timedelta
import logging

from kombu import Exchange, Queue

from django.apps import apps
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

from acls import ModelPermission
from common import (
    MayanAppConfig, menu_facet, menu_multi_item, menu_object, menu_secondary,
    menu_tools
)
from common.settings import settings_db_sync_task_delay
from documents.search import document_search, document_page_search
from documents.signals import post_version_upload
from documents.widgets import document_link
from mayan.celery import app
from navigation import SourceColumn
from rest_api.classes import APIEndPoint

from .events import event_parsing_document_version_submit
from .handlers import handler_parse_document_version
from .links import (
    link_document_content, link_document_content_download,
    link_document_parsing_errors_list, link_document_submit_multiple,
    link_document_submit, link_document_type_submit, link_error_list
)
from .permissions import permission_content_view
from .utils import get_document_content

logger = logging.getLogger(__name__)


def document_parsing_submit(self):
    latest_version = self.latest_version
    # Don't error out if document has no version
    if latest_version:
        latest_version.submit_for_parsing()


def document_version_parsing_submit(self):
    from .tasks import task_parse_document_version

    event_parsing_document_version_submit.commit(
        action_object=self.document, target=self
    )

    task_parse_document_version.apply_async(
        eta=now() + timedelta(seconds=settings_db_sync_task_delay.value),
        kwargs={'document_version_pk': self.pk},
    )


class DocumentParsingApp(MayanAppConfig):
    has_tests = True
    name = 'document_parsing'
    verbose_name = _('Document parsing')

    def ready(self):
        super(DocumentParsingApp, self).ready()

        APIEndPoint(app=self, version_string='1')

        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )

        DocumentVersion = apps.get_model(
            app_label='documents', model_name='DocumentVersion'
        )

        DocumentVersionParseError = self.get_model('DocumentVersionParseError')

        Document.add_to_class('submit_for_parsing', document_parsing_submit)
        DocumentVersion.add_to_class(
            'content', get_document_content
        )
        DocumentVersion.add_to_class(
            'submit_for_parsing', document_version_parsing_submit
        )

        ModelPermission.register(
            model=Document, permissions=(permission_content_view,)
        )

        SourceColumn(
            source=DocumentVersionParseError, label=_('Document'),
            func=lambda context: document_link(context['object'].document_version.document)
        )
        SourceColumn(
            source=DocumentVersionParseError, label=_('Added'),
            attribute='datetime_submitted'
        )
        SourceColumn(
            source=DocumentVersionParseError, label=_('Result'),
            attribute='result'
        )

        app.conf.CELERY_QUEUES.append(
            Queue('parsing', Exchange('parsing'), routing_key='parsing'),
        )

        app.conf.CELERY_ROUTES.update(
            {
                'document_parsing.tasks.task_parse_document_version': {
                    'queue': 'parsing'
                },
            }
        )

        document_search.add_model_field(
            field='versions__pages__content__content', label=_('Content')
        )

        document_page_search.add_model_field(
            field='content__content', label=_('Content')
        )

        menu_facet.bind_links(
            links=(link_document_content,), sources=(Document,)
        )
        menu_multi_item.bind_links(
            links=(link_document_submit_multiple,), sources=(Document,)
        )
        menu_object.bind_links(
            links=(link_document_submit,), sources=(Document,)
        )
        menu_secondary.bind_links(
            links=(
                link_document_content, link_document_parsing_errors_list,
                link_document_content_download
            ),
            sources=(
                'document_parsing:document_content',
                'document_parsing:document_content_download',
                'document_parsing:document_parsing_error_list',
            )
        )
        menu_tools.bind_links(
            links=(
                link_document_type_submit, link_error_list,
            )
        )

        post_version_upload.connect(
            dispatch_uid='document_parsing_handler_parse_document_version',
            receiver=handler_parse_document_version,
            sender=DocumentVersion
        )



from __future__ import absolute_import, unicode_literals

import logging
import json

import requests

from django.template import Template, Context
from django.utils.translation import ugettext_lazy as _

from .classes import WorkflowAction
from .exceptions import WorkflowStateActionError

__all__ = ('HTTPPostAction',)
logger = logging.getLogger(__name__)
DEFAULT_TIMEOUT = 4  # 4 seconds


class HTTPPostAction(WorkflowAction):
    fields = {
        'url': {
            'label': _('URL'),
            'class': 'django.forms.CharField', 'kwargs': {
                'help_text': _(
                    'Can be an IP address, a domain or a template. Templates '
                    'receive the workflow log entry instance as part of '
                    'their context via the variable "entry_log". '
                    'The "entry_log" in turn provides the '
                    '"workflow_instance", "datetime", "transition", "user", '
                    'and "comment" attributes.'
                ),
                'required': True
            },
        }, 'timeout': {
            'label': _('Timeout'),
            'class': 'django.forms.IntegerField', 'default': DEFAULT_TIMEOUT,
            'help_text': _('Time in seconds to wait for a response.'),
            'required': True

        }, 'payload': {
            'label': _('Payload'),
            'class': 'django.forms.CharField', 'kwargs': {
                'help_text': _(
                    'A JSON document to include in the request. Can also be '
                    'a template that return a JSON document. Templates '
                    'receive the workflow log entry instance as part of '
                    'their context via the variable "entry_log". '
                    'The "entry_log" in turn provides the '
                    '"workflow_instance", "datetime", "transition", "user", '
                    'and "comment" attributes.'
                ), 'required': False
            }

        },
    }
    field_order = ('url', 'timeout', 'payload')
    label = _('Perform a POST request')
    widgets = {
        'payload': {
            'class': 'django.forms.widgets.Textarea', 'kwargs': {
                'attrs': {'rows': '10'},
            }
        }
    }

    def execute(self, context):
        self.url = self.form_data.get('url')
        self.payload = self.form_data.get('payload')

        try:
            url = Template(self.url).render(
                context=Context(context)
            )
        except Exception as exception:
            raise WorkflowStateActionError(
                _('URL template error: %s') % exception
            )

        logger.debug('URL template result: %s', url)

        try:
            result = Template(self.payload or '{}').render(
                context=Context(context)
            )
        except Exception as exception:
            raise WorkflowStateActionError(
                _('Payload template error: %s') % exception
            )

        logger.debug('payload template result: %s', result)

        try:
            payload = json.loads(result, strict=False)
        except Exception as exception:
            raise WorkflowStateActionError(
                _('Payload JSON error: %s') % exception
            )

        logger.debug('payload json result: %s', payload)

        requests.post(url=url, data=payload, timeout=self.form_data['timeout'])



from __future__ import unicode_literals

from ..models import MetadataType

from .literals import TEST_METADATA_TYPE_NAME, TEST_METADATA_TYPE_LABEL


class MetadataTypeMixin(object):
    def setUp(self):
        super(MetadataTypeMixin, self).setUp()
        self.metadata_type = MetadataType.objects.create(
            name=TEST_METADATA_TYPE_NAME, label=TEST_METADATA_TYPE_LABEL
        )



from __future__ import unicode_literals

from importlib import import_module
import logging

from django.apps import apps
from django.utils import six
from django.utils.encoding import force_text

from common.classes import PropertyHelper

__all__ = ('WorkflowAction',)
logger = logging.getLogger(__name__)


class DocumentStateHelper(PropertyHelper):
    @staticmethod
    @property
    def constructor(*args, **kwargs):
        return DocumentStateHelper(*args, **kwargs)

    def get_result(self, name):
        return self.instance.workflows.get(workflow__internal_name=name)


class WorkflowActionMetaclass(type):
    _registry = {}

    def __new__(mcs, name, bases, attrs):
        new_class = super(WorkflowActionMetaclass, mcs).__new__(
            mcs, name, bases, attrs
        )

        if not new_class.__module__ == __name__:
            mcs._registry[
                '{}.{}'.format(new_class.__module__, name)
            ] = new_class

        return new_class


class WorkflowActionBase(object):
    fields = ()


class WorkflowAction(six.with_metaclass(WorkflowActionMetaclass, WorkflowActionBase)):
    @classmethod
    def clean(cls, request, form_data=None):
        return form_data

    @classmethod
    def get(cls, name):
        return cls._registry[name]

    @classmethod
    def get_all(cls):
        return sorted(cls._registry.values(), key=lambda x: x.label)

    @classmethod
    def id(cls):
        return '{}.{}'.format(cls.__module__, cls.__name__)

    @staticmethod
    def initialize():
        for app in apps.get_app_configs():
            try:
                import_module('{}.workflow_actions'.format(app.name))
            except ImportError as exception:
                if force_text(exception) not in ('No module named workflow_actions', 'No module named \'{}.workflow_actions\''.format(app.name)):
                    logger.error(
                        'Error importing %s workflow_actions.py file; %s',
                        app.name, exception
                    )

    def __init__(self, form_data=None):
        self.form_data = form_data

    def get_form_schema(self, request=None):
        result = {
            'fields': self.fields or (),
            'widgets': getattr(self, 'widgets', {})
        }

        if hasattr(self, 'field_order'):
            result['field_order'] = self.field_order

        return result



from __future__ import unicode_literals

from documents.tests.literals import TEST_DOCUMENT_FILENAME
from documents.tests.test_views import GenericDocumentViewTestCase

from ..permissions import permission_content_view
from ..utils import get_document_content


class DocumentContentViewsTestCase(GenericDocumentViewTestCase):
    _skip_file_descriptor_test = True

    # Ensure we use a PDF file
    test_document_filename = TEST_DOCUMENT_FILENAME

    def setUp(self):
        super(DocumentContentViewsTestCase, self).setUp()
        self.login_user()

    def _document_content_view(self):
        return self.get(
            'document_parsing:document_content', args=(self.document.pk,)
        )

    def test_document_content_view_no_permissions(self):
        response = self._document_content_view()

        self.assertEqual(response.status_code, 403)

    def test_document_content_view_with_permission(self):
        self.grant_permission(permission=permission_content_view)

        response = self._document_content_view()

        self.assertContains(
            response, 'Mayan EDMS Documentation', status_code=200
        )

    def test_document_parsing_download_view_no_permission(self):
        response = self.get(
            'document_parsing:document_content_download', args=(self.document.pk,)
        )

        self.assertEqual(response.status_code, 403)

    def test_download_view_with_permission(self):
        self.expected_content_type = 'application/octet-stream; charset=utf-8'

        self.grant_permission(permission=permission_content_view)
        response = self.get(
            'document_parsing:document_content_download', args=(self.document.pk,)
        )

        self.assertEqual(response.status_code, 200)

        self.assert_download_response(
            response, content=(
                ''.join(get_document_content(document=self.document))
            ),
        )



from __future__ import absolute_import, unicode_literals

from django.conf.urls import url
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from django.http import HttpResponse
from django.template import Context, Template
from django.test.utils import ContextList
from django.urls import clear_url_caches, reverse

from acls import ModelPermission
from user_management.tests import (
    TEST_ADMIN_PASSWORD, TEST_ADMIN_USERNAME, TEST_USER_USERNAME,
    TEST_USER_PASSWORD
)

from ..models import ErrorLogEntry
from ..permissions_runtime import permission_error_log_view

from .base import BaseTestCase
from .literals import TEST_VIEW_NAME, TEST_VIEW_URL

TEST_ERROR_LOG_ENTRY_RESULT = 'test_error_log_entry_result_text'


class GenericViewTestCase(BaseTestCase):
    def setUp(self):
        super(GenericViewTestCase, self).setUp()
        self.has_test_view = False

    def tearDown(self):
        from mayan.urls import urlpatterns

        self.client.logout()
        if self.has_test_view:
            urlpatterns.pop(0)
        super(GenericViewTestCase, self).tearDown()

    def add_test_view(self, test_object):
        from mayan.urls import urlpatterns

        def test_view(request):
            template = Template('{{ object }}')
            context = Context(
                {'object': test_object, 'resolved_object': test_object}
            )
            return HttpResponse(template.render(context=context))

        urlpatterns.insert(0, url(TEST_VIEW_URL, test_view, name=TEST_VIEW_NAME))
        clear_url_caches()
        self.has_test_view = True

    def get_test_view(self):
        response = self.get(TEST_VIEW_NAME)
        if isinstance(response.context, ContextList):
            # template widget rendering causes test client response to be
            # ContextList rather than RequestContext. Typecast to dictionary
            # before updating.
            result = dict(response.context).copy()
            result.update({'request': response.wsgi_request})
            return Context(result)
        else:
            response.context.update({'request': response.wsgi_request})
            return Context(response.context)

    def get(self, viewname=None, path=None, *args, **kwargs):
        data = kwargs.pop('data', {})
        follow = kwargs.pop('follow', False)

        if viewname:
            path = reverse(viewname=viewname, *args, **kwargs)

        return self.client.get(
            path=path, data=data, follow=follow
        )

    def login(self, username, password):
        logged_in = self.client.login(username=username, password=password)

        user = get_user_model().objects.get(username=username)

        self.assertTrue(logged_in)
        self.assertTrue(user.is_authenticated)

    def login_user(self):
        self.login(username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD)

    def login_admin_user(self):
        self.login(username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD)

    def logout(self):
        self.client.logout()

    def post(self, viewname=None, path=None, *args, **kwargs):
        data = kwargs.pop('data', {})
        follow = kwargs.pop('follow', False)

        if viewname:
            path = reverse(viewname=viewname, *args, **kwargs)

        return self.client.post(
            path=path, data=data, follow=follow
        )


class CommonViewTestCase(GenericViewTestCase):
    def test_about_view(self):
        self.login_user()

        response = self.get('common:about_view')
        self.assertContains(response, text='About', status_code=200)

    def _create_error_log_entry(self):
        ModelPermission.register(
            model=get_user_model(), permissions=(permission_error_log_view,)
        )
        ErrorLogEntry.objects.register(model=get_user_model())

        self.error_log_entry = self.user.error_logs.create(
            result=TEST_ERROR_LOG_ENTRY_RESULT
        )

    def _request_object_error_log_list(self):
        content_type = ContentType.objects.get_for_model(model=self.user)

        return self.get(
            'common:object_error_list', kwargs={
                'app_label': content_type.app_label,
                'model': content_type.model,
                'object_id': self.user.pk
            }, follow=True
        )

    def test_object_error_list_view_with_permissions(self):
            self._create_error_log_entry()

            self.login_user()
            self.grant_access(
                obj=self.user, permission=permission_error_log_view
            )

            response = self._request_object_error_log_list()

            self.assertContains(
                response=response, text=TEST_ERROR_LOG_ENTRY_RESULT,
                status_code=200
            )

    def test_object_error_list_view_no_permissions(self):
            self._create_error_log_entry()

            self.login_user()

            response = self._request_object_error_log_list()

            self.assertNotContains(
                response=response, text=TEST_ERROR_LOG_ENTRY_RESULT,
                status_code=403
            )



from __future__ import unicode_literals

DEFAULT_ZOOM_LEVEL = 100
DEFAULT_ROTATION = 0
DEFAULT_PAGE_NUMBER = 1
DEFAULT_PILLOW_FORMAT = 'JPEG'
DEFAULT_LIBREOFFICE_PATH = '/usr/bin/libreoffice'

DEFAULT_PDFTOPPM_DPI = 300
DEFAULT_PDFTOPPM_FORMAT = 'jpeg'  # Possible values jpeg, png, tiff
DEFAULT_PDFTOPPM_PATH = '/usr/bin/pdftoppm'
DEFAULT_PDFINFO_PATH = '/usr/bin/pdfinfo'

DIMENSION_SEPARATOR = 'x'



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from events.classes import Event

event_cabinets_add_document = Event(
    name='cabinets_add_document',
    label=_('Document added to cabinet')
)
event_cabinets_remove_document = Event(
    name='cabinets_remove_document',
    label=_('Document removed from cabinet')
)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-05-24 04:56
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('document_indexing', '0010_documentindexinstancenode_indexinstance'),
    ]

    operations = [
        migrations.AlterField(
            model_name='indextemplatenode',
            name='expression',
            field=models.TextField(
                help_text="Enter a template to render. Use Django's default "
                "templating language (https://docs.djangoproject.com/en/1.7/"
                "ref/templates/builtins/)",
                verbose_name='Indexing expression'
            ),
        ),
    ]



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from smart_settings import Namespace

namespace = Namespace(name='document_parsing', label=_('Document parsing'))

setting_pdftotext_path = namespace.add_setting(
    global_name='DOCUMENT_PARSING_PDFTOTEXT_PATH',
    default='/usr/bin/pdftotext',
    help_text=_(
        'File path to poppler\'s pdftotext program used to extract text '
        'from PDF files.'
    ),
    is_path=True
)



from __future__ import absolute_import, unicode_literals

from rest_framework import generics
from rest_framework.response import Response

from documents.models import DocumentPage
from rest_api.permissions import MayanPermission

from .models import DocumentPageContent
from .permissions import permission_content_view
from .serializers import DocumentPageContentSerializer


class APIDocumentPageContentView(generics.RetrieveAPIView):
    """
    Returns the content of the selected document page.
    ---
    GET:
        parameters:
            - name: pk
              paramType: path
              type: number
    """

    mayan_object_permissions = {
        'GET': (permission_content_view,),
    }
    permission_classes = (MayanPermission,)
    serializer_class = DocumentPageContentSerializer
    queryset = DocumentPage.objects.all()

    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()

        try:
            content = instance.content
        except DocumentPageContent.DoesNotExist:
            content = DocumentPageContent.objects.none()

        serializer = self.get_serializer(content)
        return Response(serializer.data)



from __future__ import absolute_import

from django.utils.encoding import force_text

try:
    from .local import *  # NOQA
except ImportError as exception:
    if force_text(exception) != 'No module named local' and force_text(exception) != 'No module named \'mayan.settings.local\'':
        raise
    from .base import *  # NOQA



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from common.queues import queue_tools
from task_manager.classes import CeleryQueue

queue_converter = CeleryQueue(
    name='converter', label=_('Converter'), transient=True
)
queue_documents_periodic = CeleryQueue(
    name='documents_periodic', label=_('Documents periodic'), transient=True
)
queue_uploads = CeleryQueue(
    name='uploads', label=_('Uploads')
)
queue_uploads = CeleryQueue(
    name='documents', label=_('Documents')
)

queue_documents_periodic.add_task_type(
    name='documents.tasks.task_check_delete_periods',
    label=_('Check document type delete periods')
)
queue_documents_periodic.add_task_type(
    name='documents.tasks.task_check_trash_periods',
    label=_('Check document type trash periods')
)
queue_documents_periodic.add_task_type(
    name='documents.tasks.task_delete_stubs',
    label=_('Delete document stubs')
)

queue_tools.add_task_type(
    name='documents.tasks.task_clear_image_cache',
    label=_('Clear image cache')
)

queue_converter.add_task_type(
    name='documents.tasks.task_generate_document_page_image',
    label=_('Generate document page image')
)

queue_uploads.add_task_type(
    name='documents.tasks.task_update_page_count',
    label=_('Update document page count')
)
queue_uploads.add_task_type(
    name='documents.tasks.task_upload_new_version',
    label=_('Upload new document version')
)
queue_uploads.add_task_type(
    name='documents.tasks.task_delete_document',
    label=_('Delete a document')
)



from __future__ import absolute_import, unicode_literals

from django import forms
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

from common.classes import ModelAttribute
from documents.models import Document

from .models import Index, IndexTemplateNode


class IndexListForm(forms.Form):
    indexes = forms.ModelMultipleChoiceField(
        help_text=_('Indexes to be queued for rebuilding.'),
        label=_('Indexes'), queryset=Index.objects.filter(enabled=True),
        required=False, widget=forms.widgets.CheckboxSelectMultiple()
    )


class IndexTemplateNodeForm(forms.ModelForm):
    """
    A standard model form to allow users to create a new index template node
    """
    def __init__(self, *args, **kwargs):
        super(IndexTemplateNodeForm, self).__init__(*args, **kwargs)
        self.fields['index'].widget = forms.widgets.HiddenInput()
        self.fields['parent'].widget = forms.widgets.HiddenInput()
        self.fields['expression'].help_text = ' '.join(
            [
                force_text(self.fields['expression'].help_text),
                ModelAttribute.help_text_for(
                    Document, type_names=['indexing']
                ).replace('\n', '<br>')
            ]
        )

    class Meta:
        fields = ('parent', 'index', 'expression', 'enabled', 'link_documents')
        model = IndexTemplateNode



from __future__ import unicode_literals

import sys
import types

# Useful for very coarse version differentiation.
PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3
PY34 = sys.version_info[0:2] >= (3, 4)

if PY3:
    dict_type = dict
    dictionary_type = dict
else:
    dict_type = types.DictType
    dictionary_type = types.DictionaryType

try:
    from email.Utils import collapse_rfc2231_value  # NOQA
except ImportError:
    from email.utils import collapse_rfc2231_value  # NOQA
